@@ -35,13 +35,6 @@
 
 #include "eggaccelerators.h"
 
-#if defined(__powerpc__) && defined (__linux__)
-#define USE_FBLEVEL
-#include "actions/acme-fb-level.h"
-#else
-#undef USE_FBLEVEL
-#endif
-
 #include "actions/acme.h"
 #include "actions/acme-volume.h"
 
@@ -59,9 +52,6 @@
 
 typedef struct {
 	AcmeVolume *volobj;
-#ifdef USE_FBLEVEL
-	AcmeFblevel *levobj;
-#endif
 	GladeXML *xml;
 	GtkWidget *dialog;
 	GConfClient *conf_client;
@@ -75,7 +65,6 @@ typedef struct {
 enum {
 	ICON_MUTED,
 	ICON_LOUD,
-	ICON_BRIGHT,
 	ICON_EJECT,
 };
 
@@ -151,7 +140,6 @@ do_sleep_action (char *cmd1, char *cmd2)
 static char *images[] = {
 	PIXMAPSDIR "/gnome-speakernotes-muted.png",
 	PIXMAPSDIR "/gnome-speakernotes.png",
-	PIXMAPSDIR "/acme-brightness.png",
 	PIXMAPSDIR "/acme-eject.png",
 };
 
@@ -239,6 +227,35 @@ grab_key (Acme *acme, Key *key, gboolean grab)
 	}
 }
 
+static void                                                                     
+unhookup_keysym (int keycode)
+{
+	char *command;
+
+	if (keycode <= 0)
+		return;
+
+	command = g_strdup_printf ("xmodmap -e \"keycode %d = \"", keycode);
+	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
+	g_free (command);
+}                                                                               
+
+static gboolean
+hookup_keysym (int keycode, const char *keysym)
+{
+	char *command;
+
+	if (keycode <= 0)
+		return TRUE;
+
+	command = g_strdup_printf ("xmodmap -e \"keycode %d = %s\"",
+				   keycode, keysym);
+	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
+	g_free (command);
+
+	return FALSE;
+}
+
 static gboolean
 is_valid_shortcut (const char *string)
 {
@@ -260,11 +277,11 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	g_return_if_fail (entry->key != NULL);
 
 	/* Find the key that was modified */
-	for (i = 0; i < HANDLED_KEYS; i++)
+	for (i = 0; i < PLAY_KEY; i++)
 	{
 		if (strcmp (entry->key, keys[i].gconf_key) == 0)
 		{
-			const char *tmp;
+			char *tmp;
 			Key *key;
 
 			found = TRUE;
@@ -279,20 +296,84 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 					keys[i].gconf_key, NULL);
 
 			if (is_valid_shortcut (tmp) == FALSE)
-				return;
+			{
+				g_free (tmp);
+				break;
+			}
 
 			key = g_new0 (Key, 1);
 			if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE
 			    || key->keycode == 0)
 			{
+				g_free (tmp);
 				g_free (key);
-				return;
+				break;
 			}
 
 			grab_key (acme, key, TRUE);
 			keys[i].key = key;
 
-			return;
+			g_free (tmp);
+
+			break;
+		}
+	}
+
+	if (found != FALSE)
+		return;
+
+	for (i = PLAY_KEY; i < HANDLED_KEYS; i++)
+	{
+		if (strcmp (entry->key, keys[i].gconf_key) == 0)
+		{
+			char *tmp;
+			Key *key;
+
+			if (keys[i].key != NULL)
+				unhookup_keysym (keys[i].key->keycode);
+
+			g_free (keys[i].key);
+			keys[i].key = NULL;
+
+			tmp = gconf_client_get_string (acme->conf_client,
+						       keys[i].gconf_key, NULL);
+
+			if (is_valid_shortcut (tmp) == FALSE)
+			{
+				g_free (tmp);
+				break;
+			}
+
+			key = g_new0 (Key, 1);
+			if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE
+			    || key->keycode == 0)
+			{
+				g_free (tmp);
+				g_free (key);
+				break;
+			}
+
+			switch (keys[i].key_type) {
+			case PLAY_KEY:
+				hookup_keysym (key->keycode, "XF86AudioPlay");
+				break;
+			case PAUSE_KEY:
+				hookup_keysym (key->keycode, "XF86AudioPause");
+				break; 
+			case STOP_KEY:
+				hookup_keysym (key->keycode, "XF86AudioStop");
+				break;
+			case PREVIOUS_KEY:
+				hookup_keysym (key->keycode, "XF86AudioPrev");
+				break;
+			case NEXT_KEY:                                                
+				hookup_keysym (key->keycode, "XF86AudioNext");
+				break;
+			}
+
+			keys[i].key = key;
+
+			g_free (tmp);
 		}
 	}
 }
@@ -328,7 +409,7 @@ init_kbd (Acme *acme)
 {
 	int i;
 
-	for (i = 0; i < HANDLED_KEYS; i++)
+	for (i = 0; i < PLAY_KEY; i++)
 	{
 		char *tmp;
 		Key *key;
@@ -358,6 +439,52 @@ init_kbd (Acme *acme)
 
 		grab_key (acme, key, TRUE);
 	}
+
+	for (i = PLAY_KEY; i < HANDLED_KEYS; i++)
+	{
+		char *tmp;
+		Key *key;
+
+		gconf_client_notify_add (acme->conf_client,
+					 keys[i].gconf_key, update_kbd_cb,
+					 acme, NULL, NULL);
+
+		tmp = gconf_client_get_string (acme->conf_client,
+					       keys[i].gconf_key, NULL);
+		if (!is_valid_shortcut (tmp)) {
+			g_free (tmp);
+			continue;                                           
+		}                                               
+
+		key = g_new0 (Key, 1);
+		if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE
+		    || key->keycode == 0)
+		{                                       
+			g_free (tmp);
+			g_free (key);
+			continue;
+		}                                               
+		g_free (tmp);
+
+		keys[i].key = key;
+		switch (keys[i].key_type) {
+		case PLAY_KEY:
+			hookup_keysym (keys[i].key->keycode, "XF86AudioPlay");
+			break;
+		case PAUSE_KEY:
+			hookup_keysym (keys[i].key->keycode, "XF86AudioPause");
+			break;
+		case STOP_KEY:
+			hookup_keysym (keys[i].key->keycode, "XF86AudioStop");
+			break;
+		case PREVIOUS_KEY:
+			hookup_keysym (keys[i].key->keycode, "XF86AudioPrev");
+			break;
+		case NEXT_KEY:
+			hookup_keysym (keys[i].key->keycode, "XF86AudioNext");
+			break;
+		}
+	}
 }
 
 static gboolean
@@ -544,49 +671,11 @@ do_eject_action (Acme *acme)
 	else
 		execute ("eject", TRUE);
 
+	g_free (command);
+
 	gtk_widget_set_sensitive (progress, TRUE);
 }
 
-#ifdef USE_FBLEVEL
-static void
-do_brightness_action (Acme *acme, int type)
-{
-	GtkWidget *progress;
-	int level;
-
-	if (acme->levobj == NULL)
-		return;
-
-	if (acme->dialog_timeout != 0)
-	{
-		gtk_timeout_remove (acme->dialog_timeout);
-		acme->dialog_timeout = 0;
-	}
-
-	level = acme_fblevel_get_level (acme->levobj);
-
-	dialog_init (acme);
-	acme_image_set (acme, ICON_BRIGHT);
-
-	switch (type) {
-	case BRIGHT_DOWN_KEY:
-		acme_fblevel_set_level (acme->levobj, level - 1);
-		break;
-	case BRIGHT_UP_KEY:
-		acme_fblevel_set_level (acme->levobj, level + 1);
-		break;
-	}
-
-	level = acme_fblevel_get_level (acme->levobj);
-
-	progress = glade_xml_get_widget (acme->xml, "progressbar");
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
-			(double) level / 15);
-
-	dialog_show (acme);
-}
-#endif
-
 static void
 do_sound_action (Acme *acme, int type)
 {
@@ -691,12 +780,6 @@ do_action (Acme *acme, int type)
 	case WWW_KEY:
 		do_www_action (acme, NULL);
 		break;
-#ifdef USE_FBLEVEL
-	case BRIGHT_DOWN_KEY:
-	case BRIGHT_UP_KEY:
-		do_brightness_action (acme, type);
-		break;
-#endif
 	default:
 		g_assert_not_reached ();
 	}
@@ -736,7 +819,7 @@ acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	keycode = xev->xkey.keycode;
 	state = xev->xkey.state;
 
-	for (i = 0; i < HANDLED_KEYS; i++)
+	for (i = 0; i < PLAY_KEY; i++)
 	{
 		if (keys[i].key == NULL)
 			continue;
@@ -747,10 +830,6 @@ acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 			switch (keys[i].key_type) {
 			case VOLUME_DOWN_KEY:
 			case VOLUME_UP_KEY:
-#ifdef USE_FBLEVEL
-			case BRIGHT_DOWN_KEY:
-			case BRIGHT_UP_KEY:
-#endif
 				/* auto-repeatable keys */
 				if (xev->type != KeyPress)
 					return GDK_FILTER_CONTINUE;
@@ -798,17 +877,6 @@ gnome_settings_multimedia_keys_load (GConfClient *client)
 	/* initialise Volume handler */
 	acme->volobj = acme_volume_new();
 
-#ifdef USE_FBLEVEL
-	/* initialise Frame Buffer level handler */
-	acme->levobj = acme_fblevel_new (&err);
-	if (acme->levobj == NULL && err != NULL) {
-		if (!g_error_matches (err, ACME_FBLEVEL_ERROR,
-				      ACME_FBLEVEL_ERROR_NO_POWERBOOK))
-			acme_error (err->message);
-		g_error_free (err);
-	}
-#endif
-
 	/* Start filtering the events */
 	for (l = acme->screens; l != NULL; l = l->next)
 		gdk_window_add_filter (gdk_screen_get_root_window (l->data),
